% @authors:     Ahmad Humayun,  Fuxin Li
% @contact:     ahumayun@cc.gatech.edu
% @affiliation: Georgia Institute of Technology
% @date:        Fall 2013 - Spring 2015

function precompute_pairwise_data_feature(seg_obj)
    t_pair_precomp = tic;
    
    % get the logging function
    rfprintf = seg_obj.other_params.printf_func;
    
    rfprintf('Computing pairwise data ... ');
    
    % Compute superpixel boundary edgelet information - and computes 
    % bounary difference values on superpixel boundary pixel pairs, after
    % boundary alignment
    [seg_obj.sp_data, val] = ...
        prepare_pairwise_data_feature(seg_obj.sp_data, ...
                                      seg_obj.bndry_data.bndry_thin);

    % setting median value for each edglet
    [feat, the_rest, ~, sel] = gen_sp_feats(seg_obj.sp_data, val);
    if  strncmp(seg_obj.segm_params.boundaries_method,'SketchTokens',11)
        load(fullfile(seg_obj.filepath_params.trained_models_dirpath, ...
                      'pair_trees_fix_high.mat'));
    elseif strncmp(seg_obj.segm_params.boundaries_method,'Gb',2)
        load(fullfile(seg_obj.filepath_params.trained_models_dirpath, ...
                      'pair_trees_fix_high_gb.mat'));
    elseif strncmp(seg_obj.segm_params.boundaries_method,'StructEdges',11)
        load(fullfile(seg_obj.filepath_params.trained_models_dirpath, ...
                      'pair_trees_high_StructEdges.mat'));
    else
        error(['Unknown boundary method ' ...
               seg_obj.segm_params.boundaries_method '!']);
    end
    
    accum_edge(sel) = eval_tree_lad(feat', trees, scaling_type, ...
                                    scaling, f0, rho);
    accum_edge(accum_edge < 0) = 0;
    % Somehow, the GB learned edges are of very low strength
    if strcmp(seg_obj.segm_params.boundaries_method,'Gb')
        accum_edge = accum_edge / 0.765;
    end
    % Instead of median use eval_tree_lad
    accum_edge(sel) = accum_edge(sel) * 255;
    accum_edge(~sel) = the_rest(~sel);
    seg_obj.edge_vals = accum_edge(seg_obj.sp_data.edgelet_ids);
    %draw_pairwise_sp(seg_obj.sp_data.sp_seg, bndry_pairs, edgelet_ids, accum_edge')
    time_util(seg_obj, 'precompute_pairwise_time', t_pair_precomp, 0, 1);
end
