#include <algorithm>
#include <mex.h>
#include "stdint.h"

const int NHOOD_SIZE = 8;

// Note: only works up to 65536 * 65536 image (should be enough...)
void find_unique_elems_in_neighborhood(const uint32_t *Image, const uint32_t width, const uint32_t height, uint32_t *locs, uint32_t *unique_elems)
{
	const int16_t delta_x[] = {-1,-1,-1,0,0,1,1,1};
	const int16_t delta_y[] = {-1,0,1,-1,1,-1,0,1};
	uint32_t counter = 0;
	for(uint32_t pointer=0;pointer <= width * height;pointer++)
	{
		if(Image[pointer])
			continue;
		uint32_t i,j;
		uint16_t counter2 = 0;
		locs[counter] = pointer + 1;
		i = pointer / height;
		j = pointer % height;
		for(uint16_t k=0;k<NHOOD_SIZE;k++)
		{
			uint16_t ii;
			if (i + delta_x[k] >= 0 && i + delta_x[k] < width && j + delta_y[k] >= 0 && j + delta_y[k] < height)
			{
				uint32_t to_check = Image[(i + delta_x[k]) * height + j + delta_y[k]];
				// Don't check if it's still a boundary
				if (to_check==0)
					continue;
				for (ii = 0;ii<counter2;ii++)
				{
					if (unique_elems[counter*NHOOD_SIZE + ii] == to_check)
						break;
				}
				if (ii==counter2)
				{
					unique_elems[counter*NHOOD_SIZE + counter2] = to_check;
					counter2++;
				}
			}
		}
		// Sort in ascending order
		std::sort(&unique_elems[counter*NHOOD_SIZE],&unique_elems[counter*NHOOD_SIZE+counter2]);
		counter++;
	}
}

uint32_t count_zeros(const uint32_t *image, const int32_t width, const int32_t height)
{
	uint32_t counter = 0;
	for(uint32_t pointer = 0;pointer < width*height;pointer++)
		if(image[pointer] == 0)
			counter++;
	return counter;
}

void mexFunction(int nargout, mxArray *out[], int nargin, const mxArray *in[])
{
    mwSize width, height, nsegms;
	mwSize *sizes;
	uint32_t *image, *o1, *o2, num_elems;
	    /* check argument */
    if (nargin<1) {
        mexErrMsgTxt("One input argument required ( the image in row * column form with the class uint32_t)");
    }
    if (nargout>2) {
        mexErrMsgTxt("Too many output arguments");
    }    
	sizes = (mwSize *)mxGetDimensions(in[0]);
    height = sizes[0];
    width = sizes[1];

	if (!mxIsClass(in[0],"uint32")) {
        mexErrMsgTxt("Usage: image must be a 2-dimensional uint32_t matrix");
    }
	image = (uint32_t *) mxGetData(in[0]);
	uint32_t n_elems = count_zeros(image, width, height);
    out[0] = mxCreateNumericMatrix(n_elems, 1 ,mxUINT32_CLASS, mxREAL);
    out[1] = mxCreateNumericMatrix(NHOOD_SIZE,n_elems,mxUINT32_CLASS, mxREAL);
    if (out[0]==NULL || out[1]==NULL) {
	    mexErrMsgTxt("Not enough memory for the output matrix");
    }
    o1 = (uint32_t *) mxGetData(out[0]);
    o2 = (uint32_t *) mxGetData(out[1]);

    find_unique_elems_in_neighborhood(image, width, height, o1, o2);
}