% @authors:     Ahmad Humayun
% @contact:     ahumayun@cc.gatech.edu
% @affiliation: Georgia Institute of Technology
% @date:        Fall 2013 - Summer 2014

function prep_deploy()
    % paths to various directories
    algos_dir_src = '/home/ahumayun/extern_src';
    fuxin_dir_src = '/home/ahumayun/videovolumes/fuxin_lib_src';
    
    % to be run to set the directory up for deployment
    u = userpath;
    matlab_path = u(1:end-1);

    code_root_dir = fullfile(fileparts(which(mfilename)), '..');
    extern_src_rel_dst = 'extern_src';
    prepkgd_src_rel_dst = 'prepkgd_extern_src';
    algos_dir_dst = fullfile(code_root_dir, extern_src_rel_dst);
    prepkgd_dir_dst = fullfile(code_root_dir, prepkgd_src_rel_dst);
    utils_dir_dst = fullfile(code_root_dir, 'utils');
    
    % delete old directories & files (as a result of previous run of this script)
    if_exist_del(fullfile(prepkgd_dir_dst, 'fuxin_lib_src'));
    if_exist_del(algos_dir_dst);
    if_exist_del(fullfile(utils_dir_dst, 'dFig.m'));
    if_exist_del(fullfile(utils_dir_dst, 'timerTicks.m'));
    if_exist_del(fullfile(utils_dir_dst, 'compute_error_metric.m'));
    if_exist_del(fullfile(utils_dir_dst, 'duplicateElems.m'));
    if_exist_del(fullfile(utils_dir_dst, 'mexutils.h'));
    if_exist_del(fullfile(utils_dir_dst, 'poly2mask_mex.cpp'));

    % copy files needed from matlab user directory
    copyfile(fullfile(matlab_path, 'dFig.m'), utils_dir_dst);
    copyfile(fullfile(matlab_path, 'timerTicks.m'), utils_dir_dst);
    copyfile(fullfile(matlab_path, 'duplicateElems.m'), utils_dir_dst);
    copyfile(fullfile(matlab_path, 'mexutils.h'), utils_dir_dst);
    copyfile(fullfile(matlab_path, 'poly2mask_mex.cpp'), utils_dir_dst);

    % copy necessary files from fuxin library
    mkdir(fullfile(prepkgd_dir_dst, 'fuxin_lib_src'));
    mkdir(fullfile(prepkgd_dir_dst, 'fuxin_lib_src', 'segments'));
    mkdir(fullfile(prepkgd_dir_dst, 'fuxin_lib_src', 'randfeat'));
    copyfile(fullfile(fuxin_dir_src, 'boosting'), fullfile(prepkgd_dir_dst, 'fuxin_lib_src', 'boosting'));
    copyfile(fullfile(fuxin_dir_src, 'myqueue_1.1'), fullfile(prepkgd_dir_dst, 'fuxin_lib_src', 'myqueue_1.1'));
    copyfile(fullfile(fuxin_dir_src, '@classregtree_fuxin'), fullfile(prepkgd_dir_dst, 'fuxin_lib_src', '@classregtree_fuxin'));
    copyfile(fullfile(fuxin_dir_src, 'segments', 'find_unique_elems_in_neighborhood.cpp'), fullfile(prepkgd_dir_dst, 'fuxin_lib_src', 'segments'));
    copyfile(fullfile(fuxin_dir_src, 'randfeat', 'LinearRegressor_Data.m'), fullfile(prepkgd_dir_dst, 'fuxin_lib_src', 'randfeat'));
    copyfile(fullfile(fuxin_dir_src, 'DefaultVal.m'), fullfile(prepkgd_dir_dst, 'fuxin_lib_src'));
    copyfile(fullfile(fuxin_dir_src, 'getargs.m'), fullfile(prepkgd_dir_dst, 'fuxin_lib_src'));
    
    copyfile(fullfile(algos_dir_src, 'compute_error_metric.m'), utils_dir_dst);
    
    fprintf(2, '\tRemember to copy trained_models!!\n');
end

function if_exist_del(delpath)
    if exist(delpath, 'dir')
        rmdir(delpath, 's');
    elseif exist(delpath, 'file')
        delete(delpath);
    end
end