function accP = Gen_GT_superpix_edgestr(sp_data, groundTruth, filter_match)
%     superpix_map = watershed(pb_fat);
%     myedges = superpix_map == 0;
    accP = zeros(size(sp_data.sp_seg));
    % compare to each seg in turn
    for i = 1:numel(groundTruth),
        % compute the correspondence
        [match1,match2] = correspondPixels(double(sp_data.sp_edges), double(groundTruth{i}.Boundaries));
        if exist('filter_match','var') && ~isempty(filter_match) && filter_match
        % Filter the matches based on boundary directions
        match1 = filter_matches(sp_data.sp_w_bndry, match1);
        end
        % accumulate machine matches
        accP = accP | match1;
    end
end

function match1 = filter_matches(superpix_map, match1)
    [locs, vals] = find_unique_elems_in_neighborhood(uint32(superpix_map));
    m1 = match1(locs);
    [locx, locy] = ind2sub(size(superpix_map), locs);
    sup_max = double(max(max(superpix_map)));
    % m1 reflects the ground truth mappings
    [locx2, locy2] = ind2sub(size(superpix_map), m1);
    [eigvec1_x, eigvec1_y] = compute_points_major_dir(vals, m1, locx, locy);
%    save('temp_test.mat','vals', 'eigvec1_x', 'eigvec1_y', 'locx2', 'locy2');
    locsind = filter_points_major_dir(vals(:,locx2~=0), full(eigvec1_x), full(eigvec1_y), uint32(locx2(locx2~=0)), uint32(locy2(locx2~=0)));
    l2= find(locx2~=0);
%     [eigvec2_x, eigvec2_y] = compute_points_major_dir(vals, m1, locx2, locy2);
%     % Find the matches, loose filter of not more than 60 degrees.
%     edge_match = (abs(eigvec1_x .* eigvec2_x + eigvec1_y .* eigvec2_y)) >= 0.5;
%     locsind = false(numel(locs),1);
%     for i=1:size(vals,1)
%         for j=i+1:size(vals,1)
%             nz_plc = vals(j,:) ~= 0;
%             inds = sub2ind([sup_max sup_max], vals(i,nz_plc), vals(j,nz_plc));
%             locsind(nz_plc) = locsind(nz_plc) | edge_match(inds)';
%         end
%     end
    match1(locs(l2(~locsind))) = 0;
end

function [eigvec1, eigvec2] = compute_points_major_dir(vals, m1, locx, locy)
    sup_max = double(max(max(vals)));
    Dx = sparse(sup_max, sup_max);
    Dy = sparse(sup_max, sup_max);
    Dxx = sparse(sup_max, sup_max);
    Dyy = sparse(sup_max, sup_max);
    Dxy = sparse(sup_max, sup_max);
    n = sparse(sup_max, sup_max);
    for i=1:size(vals, 1)
        for j=i+1:size(vals, 1)
            nz_plc = (vals(j,:) ~= 0) & (m1'~=0) & (locx' ~= 0);
            add_to = sub2ind([sup_max sup_max], vals(i,nz_plc), vals(j,nz_plc));
            if isempty(add_to)
                continue;
            end
            Dx(1:max(add_to)) = Dx(1:max(add_to)) + accumarray(add_to',double(locx(nz_plc)),[],[],[],true)';
            Dy(1:max(add_to))  = Dy(1:max(add_to))  + accumarray(add_to',double(locy(nz_plc)),[],[],[],true)';
            Dxx(1:max(add_to))  = Dxx(1:max(add_to))  + accumarray(add_to',double(locx(nz_plc).^2),[],[],[],true)';
            Dyy(1:max(add_to))  = Dyy(1:max(add_to))  + accumarray(add_to',double(locy(nz_plc).^2),[],[],[],true)';
            Dxy(1:max(add_to))  = Dxy(1:max(add_to))  + accumarray(add_to',double(locy(nz_plc).* locx(nz_plc)),[],[],[],true)';
            n(1:max(add_to))  = n(1:max(add_to))  + accumarray(add_to', ones(size(add_to))')';
        end
    end
    n = n + eps;
    Dx = Dx ./ n;
    Dy = Dy ./ n;
    Dxx = Dxx ./ n - Dx.^2;
    Dyy = Dyy ./ n - Dy.^2;
    Dxy = Dxy ./ n - Dx.*Dy;
    the_tr = Dxx + Dyy;
    the_det = Dxx .* Dyy - Dxy.^2;
    % we need to specifically handle diagonal situations
    sth = (abs(Dxy) > 1e-5);
    eigvec1 = sparse(size(Dx,1),size(Dx,2));
    eigvec2 = sparse(size(Dx,1),size(Dx,2));
    eig1 = the_tr(sth) ./ 2 + sqrt((the_tr(sth).^2)./4 - the_det(sth));
    sumsq = sqrt((eig1-Dyy(sth)).^2 + Dxy(sth).^2);
    eigvec1(sth) = (eig1 - Dyy(sth)) ./ sumsq;
    eigvec2(sth) = Dxy(sth) ./ sumsq;
    %Now the diagonal, do nothing if there's no x, otherwise, just find the larger element from x and y 
    onlyy = (abs(Dxy) <= 1e-5 & (Dxx > 1e-5 | Dyy > 1e-5));
    eigvec2(onlyy) = double(Dyy(onlyy) > Dxx(onlyy));
    eigvec1(onlyy) = double(Dxx(onlyy) >= Dyy(onlyy));
end
