function [bndry_thin, bndry_fat, sp_data] = ...
                                get_bndrs_plus_spx(img_filepath, varargin)
%GET_BNDRS_PLUS_SPX. Please see README.md in this directory.  This function
% uses the same functionality in rigor_obj_segments to extract boundaries 
% and spx data. It runs the method with io turned on (so no data written to 
% disk) and store_sp_edge (so it extracts spx boundary map and 
% spx + boundary map)

    % add parameters to store superpixel edge map and run IO version
    varargin = [varargin, {'io', true, 'store_sp_edge',true}];
    
    % parse and divide parameters into types
    [params_func, segm_params, filepath_params, other_params, ...
        extra_params] = parse_params(varargin{:});
    % call the parameter function to populate all parameters, and override
    % ones specified by the user
    [segm_params, filepath_params, other_params] = ...
        params_func(segm_params, filepath_params, other_params, ...
                    extra_params);
    
    % read im (replicate channels if grayscale)
    input_info.img_filepath = img_filepath;
    I = imread(img_filepath);
    if ismatrix(I), I = repmat(I, [1 1 3]); end
    
    % initialize the Segmenter (include paths, init data-structures,
    % preload data, start threads ...)
    seg_obj = Segmenter(I, segm_params, filepath_params, other_params, ...
                        input_info);
    
    % compute boundaries (and note time taken)
    bndry_filepath = return_boundaries_filepath(seg_obj);
    t_bndry = tic;
    [seg_obj.bndry_data, seg_obj.timings.extra_bndry_compute_time] = ...
        Segmenter.compute_boundaries(bndry_filepath, seg_obj.I, ...
                                     seg_obj.segm_params, ...
                                     seg_obj.other_params, ...
                                     seg_obj.preload_data);
    time_util(seg_obj, 'im_pairwise_time', t_bndry, 0, 0);
    
    % if debug, output boundaries
    diagnostic_methods('print_boundaries', seg_obj);
    
    % compute superpixels
    [seg_obj.sp_data, t_sp] = Segmenter.compute_superpixels(seg_obj.I, ...
                               seg_obj.bndry_data, seg_obj.segm_params, ...
                               seg_obj.other_params);
    time_util(seg_obj, 'superpixels_compute_time', t_sp, 0, 1);
    
    % set output data
    bndry_fat = seg_obj.bndry_data.bndry_fat;
    bndry_thin = seg_obj.bndry_data.bndry_thin;
    sp_data = seg_obj.sp_data;
end
